# Copyright (c) algovera contributors.

from collections import defaultdict

from ipywidgets import CallbackDispatcher, Widget, register
from traitlets import List, Unicode

from ._version import module_name, module_version


def _noop():
    pass


@register
class CommandPalette(Widget):
    _model_name = Unicode("CommandPaletteModel").tag(sync=True)
    _model_module = Unicode(module_name).tag(sync=True)
    _model_module_version = Unicode(module_version).tag(sync=True)

    _items = List([], read_only=True).tag(sync=True)

    def add_item(self, command_id, category, *, args=None, rank=None):
        args = args or {}
        self.send(
            {
                "func": "addItem",
                "payload": {
                    "id": command_id,
                    "category": category,
                    "args": args,
                    "rank": rank,
                },
            }
        )


@register
class CommandRegistry(Widget):
    _model_name = Unicode("CommandRegistryModel").tag(sync=True)
    _model_module = Unicode(module_name).tag(sync=True)
    _model_module_version = Unicode(module_version).tag(sync=True)

    _command_list = List(Unicode, read_only=True).tag(sync=True)
    _commands = List([], read_only=True).tag(sync=True)
    _execute_callbacks = defaultdict(_noop)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.on_msg(self._on_frontend_msg)

    def _on_frontend_msg(self, _, content, buffers):
        if content.get("event", "") == "execute":
            command_id = content.get("id")
            self._execute_callbacks[command_id]()

    def execute(self, command_id, args=None):
        args = args or {}
        self.send({"func": "execute", "payload": {"id": command_id, "args": args}})

    def list_commands(self):
        return self._command_list

    def add_command(self, command_id, execute, *, caption="", label="", icon_class=""):
        if command_id in self._command_list:
            raise Exception(f"Command {command_id} is already registered")
        # TODO: support other parameters (isEnabled, isVisible...)
        self._execute_callbacks[command_id] = execute
        self.send(
            {
                "func": "addCommand",
                "payload": {
                    "id": command_id,
                    "caption": caption,
                    "label": label,
                    "iconClass": icon_class,
                },
            }
        )

    def remove_command(self, command_id):
        # TODO: check whether to keep this method, or return disposables like in lab
        self.send({"func": "removeCommand", "payload": {"id": command_id}})



# Copyright (c) ipylab contributors.
# Distributed under the terms of the Modified BSD License.

from ipywidgets import VBox, Widget, register, widget_serialization
from traitlets import Bool, Instance, Unicode
from ._version import module_name, module_version


@register
class Title(Widget):
    _model_name = Unicode("TitleModel").tag(sync=True)
    _model_module = Unicode(module_name).tag(sync=True)
    _model_module_version = Unicode(module_version).tag(sync=True)

    label = Unicode().tag(sync=True)
    icon_class = Unicode().tag(sync=True)
    closable = Bool(True).tag(sync=True)


@register
class Panel(VBox):
    _model_name = Unicode("PanelModel").tag(sync=True)
    _model_module = Unicode(module_name).tag(sync=True)
    _model_module_version = Unicode(module_version).tag(sync=True)

    title = Instance(Title).tag(sync=True, **widget_serialization)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, title=Title(), **kwargs)


@register
class SplitPanel(Panel):
    _model_name = Unicode("SplitPanelModel").tag(sync=True)
    _model_module = Unicode(module_name).tag(sync=True)
    _model_module_version = Unicode(module_version).tag(sync=True)
    _view_name = Unicode("SplitPanelView").tag(sync=True)
    _view_module = Unicode(module_name).tag(sync=True)
    _view_module_version = Unicode(module_version).tag(sync=True)

    orientation = Unicode("vertical").tag(sync=True)

    def addWidget(self, widget):
        self.children = list(self.children) + [widget]



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) algovera contributors.

import re
import json
from pathlib import Path

__all__ = ["__version__"]


def _fetchVersion():
    HERE = Path(__file__).parent.resolve()

    for settings in HERE.rglob("package.json"):
        try:
            with settings.open() as f:
                return json.load(f)["version"]
        except FileNotFoundError:
            pass

    raise FileNotFoundError(f"Could not find package.json under dir {HERE!s}")


__version__ = _fetchVersion()

# Build up version_info tuple for backwards compatibility
pattern = r"(?P<major>\d+).(?P<minor>\d+).(?P<patch>\d+)(?P<rest>.*)"
match = re.match(pattern, __version__)
parts = [int(match[part]) for part in ["major", "minor", "patch"]]
if match["rest"]:
    parts.append(match["rest"])
version_info = tuple(parts)

module_name = "algovera"
module_version = f"^{__version__}"



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) ipylab contributors.
# Distributed under the terms of the Modified BSD License.

import asyncio

from ipywidgets import CallbackDispatcher, Widget, register, widget_serialization
from traitlets import List, Unicode, Dict

from ._version import module_name, module_version


@register
class SessionManager(Widget):
    """Expose JupyterFrontEnd.serviceManager.sessions"""

    _model_name = Unicode("SessionManagerModel").tag(sync=True)
    _model_module = Unicode(module_name).tag(sync=True)
    _model_module_version = Unicode(module_version).tag(sync=True)

    # information of the current session
    current_session = Dict(read_only=True).tag(sync=True)
    # keeps track of the list of sessions
    sessions = List([], read_only=True).tag(sync=True)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._refreshed_event = None
        self._on_refresh_callbacks = CallbackDispatcher()
        self.on_msg(self._on_frontend_msg)

    def _on_frontend_msg(self, _, content, buffers):
        if content.get("event", "") == "sessions_refreshed":
            self._refreshed_event.set()
            self._on_refresh_callbacks()

    async def refresh_running(self):
        """Force a call to refresh running sessions

        Resolved when `SessionManager.runnigSession` resolves
        """
        self.send({"func": "refreshRunning"})
        self._refreshed_event = asyncio.Event()
        await self._refreshed_event.wait()

    def running(self):
        """List all running sessions managed in the manager"""
        return self.sessions



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) ipylab contributors.
# Distributed under the terms of the Modified BSD License.

import asyncio

from ipywidgets import CallbackDispatcher, Widget, register, widget_serialization
from traitlets import Instance, Unicode
from ._version import module_name, module_version

from .commands import CommandRegistry
from .shell import Shell
from .sessions import SessionManager
from .ocean_transaction import OceanMarket


@register
class JupyterFrontEnd(Widget):
    _model_name = Unicode("JupyterFrontEndModel").tag(sync=True)
    _model_module = Unicode(module_name).tag(sync=True)
    _model_module_version = Unicode(module_version).tag(sync=True)

    version = Unicode(read_only=True).tag(sync=True)
    shell = Instance(Shell).tag(sync=True, **widget_serialization)
    commands = Instance(CommandRegistry).tag(sync=True, **widget_serialization)
    sessions = Instance(SessionManager).tag(sync=True, **widget_serialization)

    def __init__(self, *args, **kwargs):
        super().__init__(
            *args,
            shell=Shell(),
            commands=CommandRegistry(),
            sessions=SessionManager(),
            ocean = OceanMarket(),
            **kwargs
        )
        self._ready_event = asyncio.Event()
        self._on_ready_callbacks = CallbackDispatcher()
        self.on_msg(self._on_frontend_msg)
        self.private_key = ''
        self.ocean = OceanMarket()

    def _on_frontend_msg(self, _, content, buffers):
        if content.get("event", "")[0:9] == "lab_ready":
            self.private_key = content.get("event", "")[10:]
            self._ready_event.set()
            self._on_ready_callbacks()
        if self.private_key != '':
            self.ocean = OceanMarket(self.private_key)

    async def ready(self):
        await self._ready_event.wait()

    def on_ready(self, callback, remove=False):
        self._on_ready_callbacks.register_callback(callback, remove)



"""CryptoPunks Data Set"""
import os
import pickle
import numpy as np
from ocean_lib.ocean.ocean import Ocean
from ocean_lib.config import Config
from ocean_lib.web3_internal.wallet import Wallet
from ocean_lib.web3_internal.constants import ZERO_ADDRESS
from ocean_lib.common.agreements.service_types import ServiceTypes
from ocean_lib.web3_internal.currency import pretty_ether_and_wei
from ocean_lib.models.compute_input import ComputeInput
from ocean_lib.models.btoken import BToken #BToken is ERC20
from ocean_lib.web3_internal.currency import to_wei
from ocean_lib.example_config import ExampleConfig
# config = Config('config.ini')
# config = ExampleConfig.get_config()
d = {
    'network' : 'https://rinkeby.infura.io/v3/d163c48816434b0bbb3ac3925d6c6c80',
    'BLOCK_CONFIRMATIONS': 0,
   'metadataCacheUri' : 'https://aquarius.oceanprotocol.com',
   'providerUri' : 'https://provider.rinkeby.oceanprotocol.com',
   'PROVIDER_ADDRESS': '0x00bd138abd70e2f00903268f3db08f2d25677c9e',
   'downloads.path': 'consume-downloads',
}



ocean = Ocean(d)
OCEAN_token = BToken(ocean.web3, ocean.OCEAN_address)
_DATA_URL = "https://drive.google.com/file/d/1d01VQ1plsB8ZIO5VF0LKV2MxdNQjvoCW/view?usp=sharing"
_DESCRIPTION = """\
CryptoPunks is a non-fungible token (NFT) collection on the Ethereum blockchain. The dataset contains 10,000 CryptoPunk images, most of humans but also of three special types: Zombie (88), Ape (24) and Alien (9). They are provided with both clear backgrounds and teal backgrounds. 
"""
TEST_KEY = '0x5d75837394b078ce97bc289fa8d75e21000573520bfa7784a9d28ccaae602bf8'
# did = "did:op:d21196A9AC0A0Aa9df1ef6f30a440584Fe1C5E7b"
# data_token_address = '0xd21196A9AC0A0Aa9df1ef6f30a440584Fe1C5E7b'
# asset = ocean.assets.resolve(did)
# pool_address = "0x2d35D25C5BF1005B310284d00Ad05b9F35ea827B"
class OceanMarket():
    """Ocean Market integration"""
    def __init__(self, private_key=None) -> None:
        self.private_key = private_key
        self.wallet = None
    # BUY DATASET
    def buy_and_download(self, did, pool_address):
        self.wallet = Wallet(ocean.web3, private_key=self.private_key, transaction_timeout=20, block_confirmations=0)
        assert self.wallet is not None, "Wallet error, initialize app again"
        # Get asset, datatoken_address
        asset = ocean.assets.resolve(did)
        data_token_address = f'0x{did[7:]}'

        print('executing transaction')
        #my wallet
        print(f"wallet.address = '{self.wallet.address}'")
        print(f"wallet ocean = {OCEAN_token.balanceOf(self.wallet.address)}")
        print(f"wallet eth = {ocean.web3.eth.get_balance(self.wallet.address)}")
        #Verify that Bob has ETH
        assert ocean.web3.eth.get_balance(self.wallet.address) > 0, "need ganache ETH"
        #Verify that Bob has OCEAN
        assert OCEAN_token.balanceOf(self.wallet.address) > 0, "need ganache OCEAN"
        # print(f"I have {pretty_ether_and_wei(data_token.balanceOf(wallet.address), data_token.symbol())}.")
        # assert data_token.balanceOf(wallet.address) >= to_wei(1), "Bob didn't get 1.0 datatokens"
        #Bob points to the service object
        fee_receiver = ZERO_ADDRESS # could also be market address
        #Bob buys 1.0 datatokens - the amount needed to consume the dataset.
        data_token = ocean.get_data_token(data_token_address)
        ocean.pool.buy_data_tokens(
            pool_address,
            amount=to_wei(1), # buy 1.0 datatoken
            max_OCEAN_amount=to_wei(10), # pay up to 10.0 OCEAN
            from_wallet=self.wallet
        )
        print(f"I have {pretty_ether_and_wei(data_token.balanceOf(self.wallet.address), data_token.symbol())}.")
        assert data_token.balanceOf(self.wallet.address) >= to_wei(1), "I didn't get 1.0 datatokens"
        service = asset.get_service(ServiceTypes.ASSET_ACCESS)
        #Bob sends his datatoken to the service
        quote = ocean.assets.order(asset.did, self.wallet.address, service_index=service.index)
        order_tx_id = ocean.assets.pay_for_service(
                    ocean.web3,
                    quote.amount,
                    quote.data_token_address,
                    asset.did,
                    service.index,
                    fee_receiver,
                    self.wallet,
                    service.get_c2d_address()
        )
        print(f"order_tx_id = '{order_tx_id}'")
        #Bob downloads. If the connection breaks, Bob can request again by showing order_tx_id.
        file_path = ocean.assets.download(
                    asset.did,
                    service.index,
                    self.wallet,
                    order_tx_id,
                    destination='./'
        )
        print(f"file_path = '{file_path}'")
        return file_path

    def c2d(self, dataset_did, algorithm_did):
        self.wallet = Wallet(ocean.web3, private_key=self.private_key, transaction_timeout=20, block_confirmations=0)
        DATA_DDO = ocean.assets.resolve(dataset_did)  # make sure we operate on the updated and indexed metadata_cache_uri versions
        ALG_DDO = ocean.assets.resolve(algorithm_did)
        ALG_address = f'0x{algorithm_did[7:]}'

        compute_service = DATA_DDO.get_service('compute')
        algo_service = ALG_DDO.get_service('access')

        # order & pay for dataset
        print('Buying datatoken...')
        dataset_order_requirements = ocean.assets.order(
            dataset_did, self.wallet.address, service_type=compute_service.type
        )
        DATA_order_tx_id = ocean.assets.pay_for_service(
            ocean.web3,
            dataset_order_requirements.amount,
            dataset_order_requirements.data_token_address,
            dataset_did,
            compute_service.index,
            ZERO_ADDRESS,
            self.wallet,
            dataset_order_requirements.computeAddress,
        )
        print(f'Bought datatoken, tx_id: {DATA_order_tx_id}')

        # order & pay for algo
        print('Buying algorithm...')
        algo_order_requirements = ocean.assets.order(
            algorithm_did, self.wallet.address, service_type=algo_service.type
        )
        ALG_order_tx_id = ocean.assets.pay_for_service(
                ocean.web3,
                algo_order_requirements.amount,
                algo_order_requirements.data_token_address,
                algorithm_did,
                algo_service.index,
                ZERO_ADDRESS,
                self.wallet,
                algo_order_requirements.computeAddress,
        )
        print(f'Bought algorithm, tx_id: {ALG_order_tx_id}')

        compute_inputs = [ComputeInput(dataset_did, DATA_order_tx_id, compute_service.index)]
        job_id = ocean.compute.start(
            compute_inputs,
            self.wallet,
            algorithm_did=algorithm_did,
            algorithm_tx_id=ALG_order_tx_id,
            algorithm_data_token=ALG_address
        )
        print(f"Started compute job with id: {job_id}")

        print('This might take a while...')
        while ocean.compute.status(dataset_did, job_id, self.wallet)['statusText'] != 'Job finished':
            print(ocean.compute.status(dataset_did, job_id, self.wallet)['statusText'])
            pass
        result = ocean.compute.result_file(dataset_did, job_id, 0, self.wallet)
        print('Done! C2D successful.')

        return result



# Copyright (c) ipylab contributors.
# Distributed under the terms of the Modified BSD License.

from ipywidgets import Widget, register, widget_serialization
from traitlets import List, Unicode
from ._version import module_name, module_version


@register
class Shell(Widget):
    _model_name = Unicode("ShellModel").tag(sync=True)
    _model_module = Unicode(module_name).tag(sync=True)
    _model_module_version = Unicode(module_version).tag(sync=True)

    _widgets = List([], read_only=True).tag(sync=True)

    def add(self, widget, area, args=None):
        args = args or {}
        serialized_widget = widget_serialization["to_json"](widget, None)
        self.send(
            {
                "func": "add",
                "payload": {
                    "serializedWidget": serialized_widget,
                    "area": area,
                    "args": args,
                },
            }
        )

    def expand_left(self):
        self.send(
            {
                "func": "expandLeft",
                "payload": {},
            }
        )

    def expand_right(self):
        self.send(
            {
                "func": "expandRight",
                "payload": {},
            }
        )



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) ipylab contributors.
# Distributed under the terms of the Modified BSD License.

import json
from pathlib import Path

# import version first: https://github.com/pypa/setuptools/issues/1724#issuecomment-627241822
from ._version import __version__, version_info

from .jupyterfrontend import JupyterFrontEnd
from .widgets import Panel, SplitPanel

HERE = Path(__file__).parent.resolve()

with open(str(HERE / "labextension" / "package.json")) as fid:
    data = json.load(fid)


def _jupyter_labextension_paths():
    return [{"src": "labextension", "dest": data["name"]}]



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.
import json
from pathlib import Path

import setuptools

HERE = Path(__file__).parent.resolve()

# The name of the project
NAME = "algovera"

lab_path = HERE / NAME / "labextension"

# Representative files that should exist after a successful build
ensured_targets = [
    str(lab_path / "static" / "style.js"),
    str(lab_path / "package.json"),
]

data_files_spec = [
    (f"share/jupyter/labextensions/{NAME}", str(lab_path.relative_to(HERE)), "**"),
    (f"share/jupyter/labextensions/{NAME}", str(HERE), "install.json"),
]

# Get the package info from package.json
pkg_json = json.loads((HERE / "package.json").read_bytes())

try:
    from jupyter_packaging import wrap_installers, npm_builder, get_data_files

    # In develop mode, just run yarn
    builder = npm_builder(build_cmd="build", build_dir=lab_path, source_dir="src")
    cmdclass = wrap_installers(post_develop=builder, ensured_targets=ensured_targets)

    setup_args = dict(
        cmdclass=cmdclass,
        data_files=get_data_files(data_files_spec),
    )
except ImportError:
    setup_args = dict()


setup_args["version"] = pkg_json["version"]

if __name__ == "__main__":
    setuptools.setup(**setup_args)
